# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Easy creation of mock Zaqar message replies."""


def message(href='/v2/queues/dgq/messages/w78sdwsqdsib',
            ttl=0, age=0, body=None):
    body = body or {}
    return {
        'href': href,
        'ttl': ttl,
        'age': age,
        'body': body
    }
